require 'spec_helper'

describe ResourceManager::Base do

  context 'with model as class' do

    let!(:manager) {
      Class.new(ResourceManager::Base) do
        model Model
      end.new
    }

    context 'with callbacks' do
      describe '#create' do
        it 'calls before create callback' do
          result_params = nil
          callback = Proc.new{ |params| result_params = params}

          manager.class.before_create(&callback)
          manager.create(test: 1)
          expect(result_params).to eq({test: 1})
        end

        it 'calls after create callback' do
          result_params = nil
          result_item = nil
          callback = Proc.new{ |item, params| result_item = item; result_params = params}
          manager.class.after_create(&callback)

          manager.create({test: 1})

          expect(result_params).to eq({test: 1})
          expect(result_item).to be_a_kind_of(Model)
        end

        it 'returns Model instance' do
          expect(manager.create({test: 1})).to be_a_kind_of(Model)
        end

        it 'yields block' do
          manager.create({test: 1}) do |item, result|
            expect(item).to be_a_kind_of(Model)
            expect(result).to be_truthy
          end
        end

        context 'with failed saving' do
          before do
            Model.any_instance.stubs(:save).returns(false)
          end

          it 'does not call after_create' do
            result_params = nil
            result_item = nil
            callback = Proc.new{ |item, params| result_item = item; result_params = params}
            manager.class.after_create(&callback)

            manager.create({test: 1})

            expect(result_params).to be_nil
            expect(result_item).to be_nil
          end


        end

        it 'yields block' do
          _item = nil
          _result = nil
          manager.create({test: 1}) do |item, result|
            _item = item
            _result = result
          end
          expect(_item).to be_a_kind_of(Model)
          expect(_result).to be_truthy
        end
      end

      describe '#create!' do
        it 'returns Model instance' do
          expect(manager.create!({test: 1})).to be_a_kind_of(Model)
        end

        context 'with failed saving' do
          before do
            Model.any_instance.stubs(:save!).raises(StandardError.new)
          end

          it 'returns Model instance' do
            expect{manager.create!({test: 1})}.to raise_error(StandardError)
          end

        end
      end

      describe '#update' do
        it 'calls before update callback' do
          result_params = nil
          result_item = nil
          callback = Proc.new{ |item, params| result_item = item; result_params = params}
          manager.class.before_update(&callback)

          manager.update(1, test: 1)

          expect(result_params).to eq({test: 1})
          expect(result_item).to be_a_kind_of(Model)
        end


        it 'calls after update callback' do
          result_params = nil
          result_item = nil
          result_changed = nil
          callback = Proc.new{ |item, params, changed| result_item = item; result_params = params; result_changed = changed}
          manager.class.after_update(&callback)

          manager.update(1, test: 1)

          expect(result_params).to eq({test: 1})
          expect(result_item).to be_a_kind_of(Model)
          expect(result_changed).to be_a_kind_of(Hash)
        end

        it 'returns  Model instance' do
          expect(manager.update(1, {test: 1})).to be_a_kind_of(Model)
        end

        it 'yields block' do
          _item = nil
          _result = nil

          manager.update(1, {test: 1}) do |item, result|
            _item = item
            _result = result
          end

          expect(_item).to be_a_kind_of(Model)
          expect(_result).to be_truthy
        end

        context 'with failed saving' do
          before do
            Model.any_instance.stubs(:save).returns(false)
          end

          it 'does not call after_update' do
            result_params = nil
            result_item = nil
            result_changed= nil
            callback = Proc.new{ |item, params, changed| result_item = item; result_params = params; result_changed = changed}
            manager.class.after_update(&callback)

            manager.update(1, {test: 1})

            expect(result_params).to be_nil
            expect(result_item).to be_nil
            expect(result_changed).to be_nil
          end

          it 'yields block' do
            manager.update(1, {test: 1}) do |item, result|
              expect(item).to be_a_kind_of(Model)
              expect(result).to be_falsey
            end
          end
        end
      end

      describe '#update!' do
        it 'returns Model instance' do
          expect(manager.update!(1, {test: 1})).to be_a_kind_of(Model)
        end

        context 'with failed saving' do
          before do
            Model.any_instance.stubs(:save!).raises(StandardError.new)
          end

          it 'returns Model instance' do
            expect{manager.update!(1, {test: 1})}.to raise_error(StandardError)
          end

        end
      end


      describe '#destroy' do
        it 'calls before destroy callback' do
          result_item = nil
          callback = Proc.new{ |item| result_item = item}
          manager.class.before_destroy(&callback)
          manager.destroy(1)

          expect(result_item).to be_a_kind_of(Model)
        end


        it 'calls after destroy callback' do
          result_item = nil
          callback = Proc.new{ |item| result_item = item}
          manager.class.after_destroy(&callback)
          manager.destroy(1)

          expect(result_item).to be_a_kind_of(Model)
        end

        it 'returns Model instance' do
          expect(manager.destroy(Model.new)).to be_a_kind_of(Model)
        end


      end


      describe '#destroy!' do
        it 'returns Model instance' do
          expect(manager.destroy!(1)).to be_a_kind_of(Model)
        end

        context 'with failed saving' do
          before do
            Model.any_instance.stubs(:destroy!).raises(StandardError.new)
          end

          it 'returns Model instance' do
            expect{manager.destroy!(1)}.to raise_error(StandardError)
          end

        end
      end

    end

    context 'with actions' do
      describe '#create' do
        it 'calls before create callback' do
          result_params = nil
          callback = Proc.new{ |params| result_params = params}

          manager.class.create(&callback)
          manager.create(test: 1)
          expect(result_params).to eq({test: 1})
        end
      end

      describe '#update' do
        it 'calls before create callback' do
          result_params = nil
          result_item = nil
          callback = Proc.new{ |item, params| result_item = item; result_params = params}

          manager.class.update(&callback)
          manager.update(1, test: 1)
          expect(result_params).to eq({test: 1})
          expect(result_item).to be_a_kind_of(Model)
        end
      end

      describe '#destroy' do
        it 'calls before create callback' do
          result_item = nil
          callback = Proc.new{ |item| result_item = item}

          manager.class.destroy(&callback)
          manager.destroy(1)
          expect(result_item).to be_a_kind_of(Model)
        end
      end
    end
  end

  context 'with model as lambda' do
    let!(:manager) {
      Class.new(ResourceManager::Base) do
        model ->{ Model }
      end.new
    }

    describe '#create' do
      it 'Model instance' do
        expect(manager.create({test: 1})).to be_a_kind_of(Model)
      end
    end
  end
end